package com.smartgwt.sample.server;

import java.util.Date;

import com.isomorphic.datasource.DSRequest;
import com.isomorphic.datasource.DSResponse;
import com.isomorphic.log.Logger;

public class SupplyItemAuditedDMI 
{

    Logger log = new Logger(SupplyItemAuditedDMI.class.getName());

    public DSResponse fetch(DSRequest dsRequest)
        throws Exception
    {

    	// if fetch need also to be audited, can be done here
        DSResponse dsResponse = dsRequest.execute();

        return dsResponse;
    }

    public DSResponse add(DSRequest dsRequest)
        throws Exception
    {
        log.info("procesing DMI add operation");

        // execute first the sql operation
        DSResponse dsResponse = dsRequest.execute();
        
        // save the new row
        saveAudit(dsResponse, "add");
        
        return dsResponse;
    }

    public DSResponse update(DSRequest dsRequest)
        throws Exception
    {
        log.info("procesing DMI update operation");

        // execute first the sql operation
        DSResponse dsResponse = dsRequest.execute();
        
        // save the new row
        saveAudit(dsResponse, "update");
        
        return dsResponse;
    }


    public DSResponse remove(DSRequest dsRequest)
        throws Exception
    {
    	log.info("procesing DMI remove operation");
    	
    	// execute first the sql operation
        DSResponse dsResponse = dsRequest.execute();
        
        // save te new row
        saveAudit(dsResponse, "remove");
        
        return dsResponse;
    }
    
    /**
     * Save a response and a operation to the audit trail datasource
     * 
     * @param response the response to be saved
     * @param operation the operation made to get this record
     * 
     * @return the response of saving the record to audit trail database. If user wants to 
     * check the response from the server, he can check on the response returned by this
     * method 
     * 
     * @throws Exception
     */
    private DSResponse saveAudit(DSResponse response, String operation)
    		throws Exception
    {
    	DSRequest auditRequest = new DSRequest();
        auditRequest.setDataSourceName("supplyItem_AuditTrail");
        auditRequest.setValues(response.getRecord());
        auditRequest.setOperationType("add");
        
        auditRequest.setFieldValue("operationType", operation);
        auditRequest.setFieldValue("timestamp", new Date());
        
        return auditRequest.execute();
    }
}